@@ -1,6 +1,7 @@
-#[cfg(with_options)]
-use crate::fs::OpenOptions;
-use crate::fs::{Metadata, Permissions};
+use crate::{
+    fs::{Metadata, OpenOptions, Permissions},
+    fs_utf8::from_utf8,
+};
 use cap_primitives::{ambient_authority, AmbientAuthority};
 #[cfg(not(windows))]
 use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
@@ -78,8 +79,8 @@ impl File {
         self.cap_std.sync_all()
     }
 
-    /// This function is similar to `sync_all`, except that it may not synchronize
-    /// file metadata to a filesystem.
+    /// This function is similar to `sync_all`, except that it may not
+    /// synchronize file metadata to a filesystem.
     ///
     /// This corresponds to [`std::fs::File::sync_data`].
     #[inline]
@@ -87,8 +88,8 @@ impl File {
         self.cap_std.sync_data()
     }
 
-    /// Truncates or extends the underlying file, updating the size of this file
-    /// to become size.
+    /// Truncates or extends the underlying file, updating the size of this
+    /// file to become size.
     ///
     /// This corresponds to [`std::fs::File::set_len`].
     #[inline]
@@ -104,8 +105,8 @@ impl File {
         self.cap_std.metadata()
     }
 
-    /// Creates a new `File` instance that shares the same underlying file handle as the existing
-    /// `File` instance.
+    /// Creates a new `File` instance that shares the same underlying file
+    /// handle as the existing `File` instance.
     ///
     /// This corresponds to [`std::fs::File::try_clone`].
     #[inline]
@@ -120,6 +121,47 @@ impl File {
     pub fn set_permissions(&self, perm: Permissions) -> io::Result<()> {
         self.cap_std.set_permissions(perm)
     }
+
+    /// Constructs a new instance of `Self` in read-only mode by opening the
+    /// given path as a file using the host process' ambient authority.
+    ///
+    /// # Ambient Authority
+    ///
+    /// This function is not sandboxed and may access any path that the host
+    /// process has access to.
+    #[inline]
+    pub fn open_ambient<P: AsRef<str>>(
+        path: P,
+        ambient_authority: AmbientAuthority,
+    ) -> io::Result<Self> {
+        let path = from_utf8(path)?;
+        Ok(Self::from_cap_std(crate::fs::File::open_ambient(
+            path,
+            ambient_authority,
+        )?))
+    }
+
+    /// Constructs a new instance of `Self` with the options specified by
+    /// `options` by opening the given path as a file using the host process'
+    /// ambient authority.
+    ///
+    /// # Ambient Authority
+    ///
+    /// This function is not sandboxed and may access any path that the host
+    /// process has access to.
+    #[inline]
+    pub fn open_ambient_with<P: AsRef<str>>(
+        path: P,
+        options: &OpenOptions,
+        ambient_authority: AmbientAuthority,
+    ) -> io::Result<Self> {
+        let path = from_utf8(path)?;
+        Ok(Self::from_cap_std(crate::fs::File::open_ambient_with(
+            path,
+            options,
+            ambient_authority,
+        )?))
+    }
 }
 
 #[cfg(not(windows))]
@@ -469,34 +511,39 @@ impl std::os::wasi::fs::FileExt for File {
     }
 
     #[inline]
-    fn create_directory<P: AsRef<Path>>(&self, dir: P) -> std::result::Result<(), std::io::Error> {
+    fn create_directory<P: AsRef<str>>(&self, dir: P) -> std::result::Result<(), std::io::Error> {
+        let path = from_utf8(path)?;
         self.cap_std.create_directory(dir)
     }
 
     #[inline]
-    fn read_link<P: AsRef<Path>>(
+    fn read_link<P: AsRef<str>>(
         &self,
         path: P,
     ) -> std::result::Result<std::path::PathBuf, std::io::Error> {
+        let path = from_utf8(path)?;
         self.cap_std.read_link(path)
     }
 
     #[inline]
-    fn metadata_at<P: AsRef<Path>>(
+    fn metadata_at<P: AsRef<str>>(
         &self,
         lookup_flags: u32,
         path: P,
     ) -> std::result::Result<std::fs::Metadata, std::io::Error> {
+        let path = from_utf8(path)?;
         self.cap_std.metadata_at(lookup_flags, path)
     }
 
     #[inline]
-    fn remove_file<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
+    fn remove_file<P: AsRef<str>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
+        let path = from_utf8(path)?;
         self.cap_std.remove_file(path)
     }
 
     #[inline]
-    fn remove_directory<P: AsRef<Path>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
+    fn remove_directory<P: AsRef<str>>(&self, path: P) -> std::result::Result<(), std::io::Error> {
+        let path = from_utf8(path)?;
         self.cap_std.remove_directory(path)
     }
 }
